Package: rdlocrand
Type: Package
Title: Local Randomization Methods for RD Designs
Version: 2.0
Authors@R: c(person(given = c("Matias", "D."),
                      family = "Cattaneo",
                      role = c("aut", "cre"),
                      email = "matias.d.cattaneo@gmail.com"),
               person(given = "Rocio",
                      family = "Titiunik",
                      role = "aut",
                      email = "rocio.titiunik@gmail.com"),
               person(given = "Gonzalo",
                      family = "Vazquez-Bare",
                      role = "aut",
                      email = "gvazquezbare@gmail.com"))
URL: https://rdpackages.github.io/,
        https://github.com/rdpackages/rdlocrand
BugReports: https://github.com/rdpackages/rdlocrand/issues
Description: Provides tools to perform randomization inference for RD designs under local randomization: rdrandinf() to perform hypothesis testing using randomization inference, rdwinselect() to select a window around the cutoff in which randomization is likely to hold, rdsensitivity() to assess the sensitivity of the results to different window lengths and null hypotheses and rdrbounds() to construct Rosenbaum bounds for sensitivity to unobserved confounders. See Cattaneo, Titiunik and Vazquez-Bare (2016) <https://rdpackages.github.io/references/Cattaneo-Titiunik-VazquezBare_2016_Stata.pdf> for further methodological details, and references.
Imports: AER, sandwich
Depends: R (>= 3.1)
License: GPL-3
Encoding: UTF-8
NeedsCompilation: no
Author: Matias D. Cattaneo [aut, cre],
  Rocio Titiunik [aut],
  Gonzalo Vazquez-Bare [aut]
Maintainer: Matias D. Cattaneo <matias.d.cattaneo@gmail.com>
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-14 10:06:10 UTC; cattaneo
Repository: CRAN
Date/Publication: 2026-05-14 15:30:02 UTC
Built: R 4.5.2; ; 2026-05-14 18:23:04 UTC; unix
