# Generated by roxygen2: do not edit by hand

export(rdrandinf)
export(rdrbounds)
export(rdsensitivity)
export(rdwinselect)
importFrom(grDevices,gray.colors)
importFrom(graphics,filled.contour)
importFrom(graphics,title)
importFrom(stats,binom.test)
importFrom(stats,complete.cases)
importFrom(stats,cov)
importFrom(stats,ks.test)
importFrom(stats,lm)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,poly)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(stats,vcov)
